import datetime

import PyPDF2
from reportlab.pdfgen import canvas

from signpdf2.create_tmp_file import CreateTmpFileMixin
from signpdf2.pdf_utilites import PdfUtilities


class SignPdf(PdfUtilities,
              CreateTmpFileMixin):
    """
    Sign a pdf with signature image at desired location.
    we use units instead of pixel. Units are pdf-standard units (1/72 inch)
    """

    def __init__(self, sign_w: int, sign_h: int,
                 page_num: int, offset_x: int, offset_y: int,
                 pdf_file: str, signature_file: str):
        """
        :param sign_w: signature width in units
        :param sign_h: signature height in units
        :param pdf_file:  name and path of pdf file on local system
        :param signature_file: name and path of signature image file
        :param page_num: page number of pdf to sign. Index starts at 0
        :param offset_x: offset units horizontally from left
        :param offset_y: offset units vertically from bottom
        """
        self.sign_w = sign_w
        self.sign_h = sign_h
        self.page_num = page_num
        self.offset_x = offset_x
        self.offset_y = offset_y
        self.pdf_file = pdf_file
        self.signature_file = signature_file

    def sign_pdf(self, sign_date: bool = True):
        """
        Draw signature on a temporary empty single page pdf.
        Then merge this page with original pdf page. If signature is needed
        on 2nd page, then merge this temp signed page on page2.

        :param sign_date: Bool. If true, then add current timestamp below
            signature
        :return: PdfFileWriter object with signed pdf
        """
        writer = PyPDF2.PdfFileWriter()
        pdf_reader = self.get_pdf_file_reader()

        for i in range(0, pdf_reader.getNumPages()):
            orignal_pdf_page = pdf_reader.getPage(i)

            if i == self.page_num:
                temp_signature_pdf = self.create_tmp_file()

                self.draw_signature_on_pdf(
                    temp_signature_pdf,
                    page_size=orignal_pdf_page.cropBox,
                    sign_date=sign_date
                )

                # Merge signed temp PDF in to original page
                signed_pdf_reader = self.get_pdf_file_reader(temp_signature_pdf)
                signed_page = signed_pdf_reader.getPage(0)
                orignal_pdf_page = self.merge_two_pdf_pages(orignal_pdf_page,
                                                            signed_page)

            writer.addPage(orignal_pdf_page)

        return writer

    def get_pdf_file_reader(self, file: str = None):
        """
        :param file: pdf file name with path
        :return: file reader object , keeping file open in read mode. if we
        close the file, then that pdf_reader is of no use
        """
        if file is None:
            file = self.pdf_file

        pdf_file = open(file, 'rb')
        return PyPDF2.PdfFileReader(pdf_file)

    def merge_two_pdf_pages(
            self,
            page1: PyPDF2.pdf.PageObject,
            page2: PyPDF2.pdf.PageObject
    ) -> PyPDF2.pdf.PageObject:
        """
        Merge page2 in page1
        :param page1: pdf page - PyPDF2.pdf.PageObject
        :param page2: pdf page
        :return: page1 after page2 is merged in it
        """
        page2.mediaBox = page1.mediaBox
        page1.mergePage(page2)
        return page1

    def get_current_timestamp_str(self):
        return datetime.datetime.now().strftime(
            "%m-%d-%Y - %H:%M"
        )

    def draw_signature_on_pdf(
            self,
            pdf_file_name,
            page_size,
            sign_date
    ):
        """
        Draw signature on a pdf page , same size as page_size.
        Create canvas of page_size for pdf_file_name, draw signature,
        add timestamp and save it.
        :param pdf_file_name: name of pdf_file where signature is drawn
        :param page_size: size of canvas to draw on
        :param sign_date: bool - if True,add timestamp under signature
        """
        c = canvas.Canvas(pdf_file_name, pagesize=page_size)
        c.drawImage(self.signature_file, self.offset_x, self.offset_y,
                    self.sign_w, self.sign_h, mask='auto')
        if sign_date:
            timestmp = self.get_current_timestamp_str()
            c.drawString(
                self.offset_x, self.offset_y, timestmp
            )
        c.showPage()
        c.save()



import requests

from create_tmp_file import CreateTmpFileMixin


class GetFileFromUrl(CreateTmpFileMixin):
    def get_file_from_url(self, url):
        """
        :param url: url to get the file
        :return: temp filename
        """
        temp_file = self.create_tmp_file()
        with open(temp_file, "wb") as file:
            response = requests.get(url)
            file.write(response.content)

        return temp_file


class WritePdfToDisk:
    def write_pdf_to_disk(self, pdf_writer, pdf_name):
        """
        :param pdf_writer: PdfFileWriter object
        :param pdf_name: pdf file name along with entire path
        :return: write pdf to pdf_name
        """
        with open(pdf_name, 'wb') as file:
            pdf_writer.write(file)


class WritePdfToUrl(CreateTmpFileMixin,
                    WritePdfToDisk):
    def write_pdf_to_url(self, pdf_writer, url):
        """
        Upload pdf file to url using PUT req.
        We write contents of a pdf_writer to temp file and then upload that
        temp file to url.
        :param pdf_writer: PdfFileWriter object
        :param url: PUT url to make a put req with file as data.
        :return: None
        """
        temp_file = self.create_tmp_file()
        self.write_pdf_to_disk(pdf_writer, pdf_name=temp_file)

        with open(temp_file, 'rb') as tf:
            requests.put(url, data=tf)



from file_utilities import GetFileFromUrl, WritePdfToDisk, WritePdfToUrl
from sign_pdf import SignPdf

put_url = 'xyz/xyz'
pdfurl = 'https:/xyz.pdf'
sign_url = 'https://xyz.png'
output_file_name = 'signed_pdf.pdf'

pdf_file_name = GetFileFromUrl().get_file_from_url(pdfurl)
signature_file_name = GetFileFromUrl().get_file_from_url(sign_url)

sign_pdf = SignPdf(
    sign_w=100,
    sign_h=60,
    page_num=0,
    offset_x=400,
    offset_y=200,
    pdf_file=pdf_file_name,
    signature_file=signature_file_name
)
pdf_writer = sign_pdf.sign_pdf()
WritePdfToUrl().write_pdf_to_url(pdf_writer, put_url)
WritePdfToDisk().write_pdf_to_disk(pdf_writer, output_file_name)



from pdfminer import high_level
from pdfminer.pdfdocument import PDFDocument
from pdfminer.pdfinterp import resolve1
from pdfminer.pdfparser import PDFParser


class PdfUtilities:

    def get_total_number_of_pages(self, pdf_file_name):
        with open(pdf_file_name, 'rb') as file:
            parser = PDFParser(file)
            document = PDFDocument(parser)
            total_pages = resolve1(document.catalog['Pages'])['Count']

        return total_pages

    def get_text_for_a_page(self, pdf_file_name, page_num):
        text = high_level.extract_text(
            pdf_file_name, "", page_num
        )
        return text

    def get_page(self, pdf_file_name, page_num):
        return high_level.extract_pages(pdf_file_name, page_numbers=page_num)



import pathlib

from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="signpdf2",
    version="3.0.0",
    description="Read the latest Real Python tutorials",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/aseem-hegshetye/signpdf",
    author="Aseem Hegshetye",
    author_email="aseem.hegshetye@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    packages=["signpdf2"],
    include_package_data=True,
    install_requires=["PyPDF2", "requests", "urllib3", "reportlab"],

)



import tempfile


class CreateTmpFileMixin:
    def create_tmp_file(self, suffix=".pdf"):
        """
        :param suffix: temp file extention
        :return: create a temp file and return its name with entire path
        """
        with tempfile.NamedTemporaryFile(suffix=suffix) as fh:
            return fh.name
